@@ -1,11 +1,12 @@
 import logging
 import random
 
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, cast
 
 from pydantic import BaseModel, Field
 
 from .builders import ConversationBuilder
+from .exceptions import DataSetGeneratorError
 from .progress import ProgressReporter
 from .schemas import (
     AgentContext,
@@ -142,7 +143,7 @@ class SingleTurnAgentBuilder(ConversationBuilder):
 
         # Assemble into Conversation
         return self._build_conversation(
-            user_message, reasoning, tool_calls, tool_results, agent_response
+            user_message, reasoning, tool_calls, tool_results, agent_response, topic_prompt
         )
 
     async def _generate_user_question(self, topic_prompt: str) -> ChatMessage:
@@ -167,23 +168,27 @@ Example format: "Can you tell me the weather in Paris tomorrow and suggest what
 
 Generate only the user's question:"""
 
-        response = None
+        response: UserQuestion
         if self.progress_reporter:
+            temp_response = None
             async for chunk, result in self.llm.generate_async_stream(
                 prompt=prompt,
                 schema=UserQuestion,
-                max_tokens=1000,
+                max_tokens=self.config.max_tokens,
                 temperature=self.config.temperature,
             ):
                 if chunk:
                     self.progress_reporter.emit_chunk("user_question", chunk)
                 if result:
-                    response = result
+                    temp_response = result
+            if temp_response is None:
+                raise DataSetGeneratorError("Failed to generate user question")
+            response = cast(UserQuestion, temp_response)
         else:
             response = await self.llm.generate_async(
                 prompt=prompt,
                 schema=UserQuestion,
-                max_tokens=1000,
+                max_tokens=self.config.max_tokens,
                 temperature=self.config.temperature,
             )
 
@@ -217,23 +222,28 @@ Provide step-by-step reasoning (2-4 steps) and identify which tools to call with
             reasoning_steps: list[ReasoningStep]
             tool_calls: list[ToolExecution]
 
-        response = None
+        response: AgentThinking
         if self.progress_reporter:
+            temp_response = None
             async for chunk, result in self.llm.generate_async_stream(
                 prompt=prompt,
                 schema=AgentThinking,
-                max_tokens=2000,
+                max_tokens=self.config.max_tokens,
                 temperature=self.config.temperature,
             ):
                 if chunk:
                     self.progress_reporter.emit_chunk("agent_reasoning", chunk)
                 if result:
-                    response = result
+                    temp_response = result
+            if temp_response is None:
+                msg = "Failed to generate agent thinking"
+                raise DataSetGeneratorError(msg)
+            response = cast(AgentThinking, temp_response)
         else:
             response = await self.llm.generate_async(
                 prompt=prompt,
                 schema=AgentThinking,
-                max_tokens=2000,
+                max_tokens=self.config.max_tokens,
                 temperature=self.config.temperature,
             )
 
@@ -268,12 +278,13 @@ Arguments called with: {tool_call.arguments}
 
 Generate the tool's output/result. Make it realistic and appropriate for the tool and arguments."""
 
-            result = None
+            result: ToolOutput
             if self.progress_reporter:
+                temp_result = None
                 async for chunk, res in self.llm.generate_async_stream(
                     prompt=prompt,
                     schema=ToolOutput,
-                    max_tokens=1000,
+                    max_tokens=self.config.max_tokens,
                     temperature=0.7,
                 ):
                     if chunk:
@@ -281,12 +292,16 @@ Generate the tool's output/result. Make it realistic and appropriate for the too
                             f"tool_sim_{tool_call.function_name}", chunk
                         )
                     if res:
-                        result = res
+                        temp_result = res
+                if temp_result is None:
+                    msg = f"Failed to generate tool output for {tool_call.function_name}"
+                    raise DataSetGeneratorError(msg)
+                result = cast(ToolOutput, temp_result)
             else:
                 result = await self.llm.generate_async(
                     prompt=prompt,
                     schema=ToolOutput,
-                    max_tokens=1000,
+                    max_tokens=self.config.max_tokens,
                     temperature=0.7,  # Slightly creative for variety
                 )
 
@@ -332,23 +347,28 @@ You executed these tools:
 
 Based on these results, provide a clear, helpful response to the user."""
 
-        response = None
+        response: AgentResponse
         if self.progress_reporter:
+            temp_response = None
             async for chunk, result in self.llm.generate_async_stream(
                 prompt=prompt,
                 schema=AgentResponse,
-                max_tokens=800,
+                max_tokens=self.config.max_tokens,
                 temperature=self.config.temperature,
             ):
                 if chunk:
                     self.progress_reporter.emit_chunk("agent_response", chunk)
                 if result:
-                    response = result
+                    temp_response = result
+            if temp_response is None:
+                msg = "Failed to generate agent response"
+                raise DataSetGeneratorError(msg)
+            response = cast(AgentResponse, temp_response)
         else:
             response = await self.llm.generate_async(
                 prompt=prompt,
                 schema=AgentResponse,
-                max_tokens=800,
+                max_tokens=self.config.max_tokens,
                 temperature=self.config.temperature,
             )
 
@@ -361,6 +381,7 @@ Based on these results, provide a clear, helpful response to the user."""
         _tool_calls: list[ToolExecution],
         tool_results: list[ToolExecution],
         agent_response: ChatMessage,
+        topic_prompt: str = "",
     ) -> Conversation:
         """Assemble all components into a Conversation.
 
@@ -370,17 +391,16 @@ Based on these results, provide a clear, helpful response to the user."""
             _tool_calls: Tool calls made (unused, kept for interface consistency)
             tool_results: Tool execution results (contains completed tool calls with results)
             agent_response: Agent's final response
+            topic_prompt: Topic used to generate this conversation (for metadata)
 
         Returns:
             Complete Conversation object
         """
         messages = []
 
-        # Add system message if configured
-        if self.config.sys_msg:
-            messages.append(
-                ChatMessage(role="system", content=self.config.dataset_system_prompt or "")
-            )
+        # Don't add system message for agent mode - it interferes with tool calling
+        # The system prompt teaches models to explain tool usage instead of executing tools
+        # For tool calling, the tool definitions themselves serve as instructions
 
         # Add user message
         messages.append(user_message)
@@ -411,6 +431,12 @@ Based on these results, provide a clear, helpful response to the user."""
         # Build agent context
         agent_context = AgentContext(mode="single_turn")
 
+        # Build metadata
+        metadata = {
+            "conversation_type": "chain_of_thought" if reasoning_trace else "basic",
+            "topic": topic_prompt if topic_prompt else "general",
+        }
+
         return Conversation(
             messages=messages,
             reasoning=reasoning_trace,
@@ -418,6 +444,7 @@ Based on these results, provide a clear, helpful response to the user."""
             agent_context=agent_context,
             question=user_message.content,  # Set question field for formatters
             final_answer=agent_response.content,  # Set final_answer field for formatters
+            metadata=metadata,
         )
 
     def _format_tools_for_prompt(self) -> str:
@@ -484,7 +511,7 @@ class MultiTurnAgentBuilder(SingleTurnAgentBuilder):
                 break
 
         # Assemble into complete conversation
-        return self._build_multi_turn_conversation(turns, scenario)
+        return self._build_multi_turn_conversation(turns, scenario, topic_prompt)
 
     async def _generate_scenario(self, topic_prompt: str, num_turns: int) -> str:
         """Generate a multi-turn scenario description.
@@ -506,23 +533,28 @@ The scenario should:
 
 Keep it brief (2-3 sentences)."""
 
-        response = None
+        response: Scenario
         if self.progress_reporter:
+            temp_response = None
             async for chunk, result in self.llm.generate_async_stream(
                 prompt=prompt,
                 schema=Scenario,
-                max_tokens=1000,
+                max_tokens=self.config.max_tokens,
                 temperature=self.config.temperature,
             ):
                 if chunk:
                     self.progress_reporter.emit_chunk("scenario_gen", chunk)
                 if result:
-                    response = result
+                    temp_response = result
+            if temp_response is None:
+                msg = "Failed to generate scenario"
+                raise DataSetGeneratorError(msg)
+            response = cast(Scenario, temp_response)
         else:
             response = await self.llm.generate_async(
                 prompt=prompt,
                 schema=Scenario,
-                max_tokens=1000,
+                max_tokens=self.config.max_tokens,
                 temperature=self.config.temperature,
             )
 
@@ -606,12 +638,13 @@ Guidance: {guidance}
 The message should reference or build upon previous conversation if applicable.
 Keep it concise and natural."""
 
-        response = None
+        response: UserQuestion
         if self.progress_reporter:
+            temp_response = None
             async for chunk, result in self.llm.generate_async_stream(
                 prompt=prompt,
                 schema=UserQuestion,
-                max_tokens=150,
+                max_tokens=self.config.max_tokens,
                 temperature=self.config.temperature,
             ):
                 if chunk:
@@ -619,12 +652,16 @@ Keep it concise and natural."""
                         f"turn_{turn_idx}_user", chunk, turn=turn_idx + 1
                     )
                 if result:
-                    response = result
+                    temp_response = result
+            if temp_response is None:
+                msg = f"Failed to generate user question for turn {turn_idx + 1}"
+                raise DataSetGeneratorError(msg)
+            response = cast(UserQuestion, temp_response)
         else:
             response = await self.llm.generate_async(
                 prompt=prompt,
                 schema=UserQuestion,
-                max_tokens=150,
+                max_tokens=self.config.max_tokens,
                 temperature=self.config.temperature,
             )
 
@@ -661,23 +698,28 @@ Provide step-by-step reasoning (1-3 steps) and identify which tools are needed."
             reasoning_steps: list[ReasoningStep]
             tool_calls: list[ToolExecution]
 
-        response = None
+        response: AgentThinking
         if self.progress_reporter:
+            temp_response = None
             async for chunk, result in self.llm.generate_async_stream(
                 prompt=prompt,
                 schema=AgentThinking,
-                max_tokens=2000,
+                max_tokens=self.config.max_tokens,
                 temperature=self.config.temperature,
             ):
                 if chunk:
                     self.progress_reporter.emit_chunk("agent_thinking_mt", chunk)
                 if result:
-                    response = result
+                    temp_response = result
+            if temp_response is None:
+                msg = "Failed to generate agent thinking for multi-turn"
+                raise DataSetGeneratorError(msg)
+            response = cast(AgentThinking, temp_response)
         else:
             response = await self.llm.generate_async(
                 prompt=prompt,
                 schema=AgentThinking,
-                max_tokens=2000,
+                max_tokens=self.config.max_tokens,
                 temperature=self.config.temperature,
             )
 
@@ -736,24 +778,23 @@ Is the user's original task/goal from the scenario fully completed?
         return "\n".join(lines)
 
     def _build_multi_turn_conversation(
-        self, turns: list[AgentTurnData], scenario: str
+        self, turns: list[AgentTurnData], scenario: str, topic_prompt: str = ""
     ) -> Conversation:
         """Assemble multi-turn conversation from turn data.
 
         Args:
             turns: List of turn data
             scenario: Scenario description
+            topic_prompt: Topic used to generate this conversation (for metadata)
 
         Returns:
             Complete Conversation object
         """
         messages = []
 
-        # Add system message if configured
-        if self.config.sys_msg:
-            messages.append(
-                ChatMessage(role="system", content=self.config.dataset_system_prompt or "")
-            )
+        # Don't add system message for agent mode - it interferes with tool calling
+        # The system prompt teaches models to explain tool usage instead of executing tools
+        # For tool calling, the tool definitions themselves serve as instructions
 
         # Collect all reasoning steps and tool executions
         all_reasoning: list[ReasoningStep] = []
@@ -799,9 +840,16 @@ Is the user's original task/goal from the scenario fully completed?
             execution_summary=f"Completed {len(turns)}-turn conversation",
         )
 
+        # Build metadata
+        metadata = {
+            "conversation_type": "chain_of_thought" if reasoning_trace else "basic",
+            "topic": topic_prompt if topic_prompt else "general",
+        }
+
         return Conversation(
             messages=messages,
             reasoning=reasoning_trace,
             tool_context=tool_context,
             agent_context=agent_context,
+            metadata=metadata,
         )
