@@ -770,6 +770,384 @@ def info() -> None:
         sys.exit(1)
 
 
+@cli.command()
+@click.argument("dataset_path", type=click.Path(exists=True))
+@click.option(
+    "--train-output",
+    type=click.Path(),
+    help="Output path for training set (required for jsonl format)",
+)
+@click.option(
+    "--eval-output",
+    type=click.Path(),
+    help="Output path for evaluation set (required for jsonl format)",
+)
+@click.option(
+    "--output-format",
+    type=click.Choice(["jsonl", "hf_dataset"]),
+    default="jsonl",
+    help="Output format: jsonl (separate files) or hf_dataset (DatasetDict)",
+)
+@click.option(
+    "--hf-repo",
+    type=str,
+    help="HuggingFace Hub repository to push to (format: username/dataset-name)",
+)
+@click.option(
+    "--test-size",
+    default=0.2,
+    type=float,
+    help="Fraction for eval set (default: 0.2)",
+)
+@click.option(
+    "--stratify-by",
+    type=click.Choice(["topic", "tool", "conversation_type"]),
+    help="Field to stratify by for balanced splitting",
+)
+@click.option(
+    "--seed",
+    default=42,
+    type=int,
+    help="Random seed for reproducibility (default: 42)",
+)
+@click.option(
+    "--shuffle/--no-shuffle",
+    default=True,
+    help="Shuffle before splitting (default: shuffle)",
+)
+def split(
+    dataset_path: str,
+    train_output: str | None,
+    eval_output: str | None,
+    output_format: str,
+    hf_repo: str | None,
+    test_size: float,
+    stratify_by: str | None,
+    seed: int,
+    shuffle: bool,
+) -> None:
+    """Split dataset into train/eval sets with optional stratification.
+
+    This command splits a DeepFabric dataset into training and evaluation sets,
+    preserving all conversation structure and metadata. Stratification ensures
+    balanced representation across the specified field.
+
+    Examples:
+
+        \b
+        # Split to JSONL files with 20% for evaluation, stratified by topic
+        deepfabric split dataset.jsonl \\
+            --train-output train.jsonl \\
+            --eval-output eval.jsonl \\
+            --test-size 0.2 \\
+            --stratify-by topic
+
+        \b
+        # Split and push to HuggingFace Hub
+        deepfabric split dataset.jsonl \\
+            --output-format hf_dataset \\
+            --hf-repo username/my-dataset \\
+            --test-size 0.2 \\
+            --stratify-by topic
+    """
+    tui = get_tui()
+
+    try:
+        from .evaluation import SplitConfig, split_dataset, split_to_hf_dataset  # noqa: PLC0415
+
+        # Validate arguments based on output format
+        if output_format == "jsonl" and (not train_output or not eval_output):
+            tui.error("--train-output and --eval-output are required for jsonl format")
+            sys.exit(1)
+        if output_format == "hf_dataset" and not hf_repo:
+            tui.error("--hf-repo is required for hf_dataset format")
+            sys.exit(1)
+
+        # Create configuration
+        config = SplitConfig(
+            test_size=test_size,
+            stratify_by=stratify_by,  # type: ignore[arg-type]
+            seed=seed,
+            shuffle=shuffle,
+        )
+
+        # Display configuration
+        tui.console.print(f"\n[bold]Splitting dataset:[/bold] {dataset_path}")
+        tui.console.print(f"  Output format: {output_format}")
+        tui.console.print(f"  Test size: {test_size * 100:.1f}%")
+        if stratify_by:
+            tui.console.print(f"  Stratify by: {stratify_by}")
+        tui.console.print(f"  Random seed: {seed}")
+        tui.console.print(f"  Shuffle: {'yes' if shuffle else 'no'}\n")
+
+        if output_format == "jsonl":
+            # JSONL output format
+            result = split_dataset(
+                dataset_path=dataset_path,
+                train_output=train_output,  # type: ignore[arg-type]
+                eval_output=eval_output,  # type: ignore[arg-type]
+                config=config,
+            )
+
+            # Display results
+            tui.success("Split complete!")
+            tui.console.print("\n[bold green]Results:[/bold green]")
+            tui.console.print(f"  Train: {result.train_size} samples → {train_output}")
+            tui.console.print(f"  Eval:  {result.eval_size} samples → {eval_output}")
+            tui.console.print(f"  Total: {result.total_size} samples")
+
+        else:
+            # HuggingFace Dataset format
+            dataset_dict, result = split_to_hf_dataset(
+                dataset_path=dataset_path,
+                config=config,
+            )
+
+            # Display results
+            tui.success("Split complete!")
+            tui.console.print("\n[bold green]Results:[/bold green]")
+            tui.console.print(f"  Train: {result.train_size} samples")
+            tui.console.print(f"  Test:  {result.eval_size} samples")
+            tui.console.print(f"  Total: {result.total_size} samples")
+
+            # Push to HuggingFace Hub
+            tui.console.print(f"\n[bold]Pushing to HuggingFace Hub:[/bold] {hf_repo}")
+            dataset_dict.push_to_hub(hf_repo)
+            tui.success(f"Successfully pushed dataset to {hf_repo}")
+            tui.console.print(f"  View at: https://huggingface.co/datasets/{hf_repo}")
+
+        # Display stratification distribution if used
+        if result.strata_distribution:
+            tui.console.print(f"\n[bold]Stratification by {stratify_by}:[/bold]")
+            for stratum, counts in result.strata_distribution.items():
+                tui.console.print(
+                    f"  {stratum}: {counts['train']} train, {counts['eval']} eval "
+                    f"(total: {counts['total']})"
+                )
+
+    except ValueError as e:
+        handle_error(click.get_current_context(), e)
+    except FileNotFoundError as e:
+        handle_error(click.get_current_context(), e)
+    except Exception as e:
+        handle_error(click.get_current_context(), e)
+
+
+@cli.command()
+@click.argument("model_path", type=click.Path())
+@click.argument("dataset_path", type=click.Path(exists=True))
+@click.option(
+    "--output",
+    "-o",
+    type=click.Path(),
+    help="Path to save evaluation results (JSON)",
+)
+@click.option(
+    "--adapter-path",
+    type=click.Path(),
+    help="Path to PEFT/LoRA adapter (for adapter-based fine-tuning)",
+)
+@click.option(
+    "--batch-size",
+    type=int,
+    default=1,
+    help="Batch size for evaluation",
+)
+@click.option(
+    "--max-samples",
+    type=int,
+    help="Maximum number of samples to evaluate (default: all)",
+)
+@click.option(
+    "--temperature",
+    type=float,
+    default=0.7,
+    help="Sampling temperature",
+)
+@click.option(
+    "--max-tokens",
+    type=int,
+    default=2048,
+    help="Maximum tokens to generate",
+)
+@click.option(
+    "--top-p",
+    type=float,
+    default=0.9,
+    help="Nucleus sampling top-p",
+)
+@click.option(
+    "--backend",
+    type=click.Choice(["transformers", "ollama"]),
+    default="transformers",
+    help="Inference backend to use",
+)
+@click.option(
+    "--device",
+    type=str,
+    help="Device to use (cuda, cpu, mps, etc.) - only for transformers backend",
+)
+@click.option(
+    "--no-save-predictions",
+    is_flag=True,
+    help="Don't save individual predictions to output file",
+)
+def evaluate(
+    model_path: str,
+    dataset_path: str,
+    output: str | None,
+    adapter_path: str | None,
+    batch_size: int,
+    max_samples: int | None,
+    temperature: float,
+    max_tokens: int,
+    top_p: float,
+    backend: str,
+    device: str | None,
+    no_save_predictions: bool,
+):
+    """Evaluate a fine-tuned model on tool-calling tasks.
+
+    MODEL_PATH: Path to base model or fine-tuned model (local directory or HuggingFace Hub ID)
+
+    DATASET_PATH: Path to evaluation dataset (JSONL format)
+
+    Typical workflow:
+
+        # Full fine-tuning: evaluate checkpoint
+        deepfabric evaluate ./checkpoints/final ./eval.jsonl --output results.json
+
+        # LoRA/PEFT: evaluate adapter on base model
+        deepfabric evaluate unsloth/Qwen3-4B-Instruct ./eval.jsonl \\
+            --adapter-path ./lora_model \\
+            --output results.json
+
+        # Quick evaluation during development
+        deepfabric evaluate ./my-model ./eval.jsonl --max-samples 50
+
+        # Evaluate HuggingFace model
+        deepfabric evaluate username/model-name ./eval.jsonl \\
+            --temperature 0.5 \\
+            --device cuda
+    """
+    tui = get_tui()
+
+    try:
+        from typing import Literal, cast  # noqa: PLC0415
+
+        from .evaluation import EvaluatorConfig, InferenceConfig  # noqa: PLC0415
+        from .evaluation.evaluator import Evaluator  # noqa: PLC0415
+
+        # Create inference configuration
+        inference_config = InferenceConfig(
+            model_path=model_path,
+            adapter_path=adapter_path,
+            backend=cast(Literal["transformers", "ollama"], backend),
+            temperature=temperature,
+            max_tokens=max_tokens,
+            top_p=top_p,
+            device=device,
+            batch_size=batch_size,
+        )
+
+        # Create evaluator configuration
+        evaluator_config = EvaluatorConfig(
+            model_path=model_path,
+            dataset_path=dataset_path,
+            output_path=output,
+            inference_config=inference_config,
+            batch_size=batch_size,
+            max_samples=max_samples,
+            save_predictions=not no_save_predictions,
+        )
+
+        # Display configuration
+        tui.console.print("\n[bold]Evaluation Configuration:[/bold]")
+        tui.console.print(f"  Model: {model_path}")
+        tui.console.print(f"  Backend: {backend}")
+        if adapter_path:
+            tui.console.print(f"  Adapter: {adapter_path}")
+        tui.console.print(f"  Dataset: {dataset_path}")
+        if output:
+            tui.console.print(f"  Output: {output}")
+        tui.console.print(f"  Batch size: {batch_size}")
+        if max_samples:
+            tui.console.print(f"  Max samples: {max_samples}")
+        tui.console.print(f"  Temperature: {temperature}")
+        tui.console.print(f"  Max tokens: {max_tokens}")
+        if device and backend == "transformers":
+            tui.console.print(f"  Device: {device}")
+        tui.console.print()
+
+        # Create evaluator and run evaluation
+        tui.console.print("[bold blue]Loading model...[/bold blue]")
+        tui.console.print(
+            "[dim]This may take several minutes for large models (downloading + loading into memory)[/dim]"
+        )
+        evaluator = Evaluator(evaluator_config)
+        tui.console.print("[green]Model loaded successfully![/green]\n")
+
+        # Track evaluation start
+        trace(
+            "evaluation_started",
+            {
+                "model_path": model_path,
+                "backend": backend,
+                "has_adapter": adapter_path is not None,
+                "dataset_path": dataset_path,
+                "batch_size": batch_size,
+                "max_samples": max_samples,
+                "temperature": temperature,
+            },
+        )
+
+        try:
+            result = evaluator.evaluate()
+
+            # Print summary
+            evaluator.print_summary(result.metrics)
+
+            if output:
+                tui.console.print(f"\n[green]Full results saved to {output}[/green]")
+
+        finally:
+            evaluator.cleanup()
+
+    except FileNotFoundError as e:
+        trace(
+            "evaluation_failed",
+            {
+                "model_path": model_path,
+                "backend": backend,
+                "dataset_path": dataset_path,
+                "error_type": "FileNotFoundError",
+            },
+        )
+        handle_error(click.get_current_context(), e)
+    except ValueError as e:
+        trace(
+            "evaluation_failed",
+            {
+                "model_path": model_path,
+                "backend": backend,
+                "dataset_path": dataset_path,
+                "error_type": "ValueError",
+            },
+        )
+        handle_error(click.get_current_context(), e)
+    except Exception as e:
+        trace(
+            "evaluation_failed",
+            {
+                "model_path": model_path,
+                "backend": backend,
+                "dataset_path": dataset_path,
+                "error_type": type(e).__name__,
+            },
+        )
+        handle_error(click.get_current_context(), e)
+
+
 # Add the format command to the CLI group
 cli.add_command(format_cli)
 
